------------------------------------------------------------------------------------------------------------------
-- Import
----------------------------------------------------------------------------------------------------

local classHelper  = require("scenario/intersection/classHelper")
local timerHandler = require("scenario/intersection/handlers/timerHandler")

local ClassStandardExplosion = require("scenario/intersection/classes/explosions/classStandardExplosion")
local ClassCrashbreaker      = require("scenario/intersection/classes/crashbreaker/classCrashbreaker")

------------------------------------------------------------------------------------------------------------------

local ClassStandardCrashbreaker = classHelper.inheritsFrom(ClassCrashbreaker)

------------------------------------------------------------------------------------------------------------------
-- Instance Methods
----------------------------------------------------------------------------------------------------

function ClassStandardCrashbreaker:update(dt)
    --[[
    Updates the explosion effect
    Parameters:
        dt - the time since last frame
    ]]--
    if not self.locked and not self.used then
        self.used = self.explosion:update(dt)
    end
end

function ClassStandardCrashbreaker:instanceOf()
    return "ClassStandardCrashbreaker"
end

-------------------------------------------------------------------------------------------------------
-- Overridden Methods
-- For method documentation, see superclass
----------------------------------------------------------------------------------------------------

function ClassStandardCrashbreaker:use(vehicle)
    --[[
    Begins the usage sequence
    Parameters:
        vehicle - the vehicle triggering the crashbreaker
    ]]--
    self.explosion:use(vehicle)

    self.inUse = true

    timerHandler.initCrashtime()
end

function ClassStandardCrashbreaker:reset()
    ClassCrashbreaker.reset(self)
    self.explosion:reset()
end

function ClassStandardCrashbreaker:unlock()
    ClassCrashbreaker.unlock(self)

    if self.isUILinked then
        guihooks.trigger("i_messages_enqueue", {
            text = "CRASHBREAKER READY",
            cssClass = "flame",
            relPath = "pickups/crashbreaker.png",
            duration = 5,
            colour = "Yellow"
        })
    end
end

-------------------------------------------------------------------------------------------------------
-- Constructor
----------------------------------------------------------------------------------------------------

function ClassStandardCrashbreaker:new(vehicles, vehicleRequirement, isUILinked)
    --[[
    See ClassCrashbreaker:new()
    ]]--
    local self = ClassCrashbreaker:new(vehicles, vehicleRequirement, isUILinked)

    self.explosion = ClassStandardExplosion:new(vehicles)

    setmetatable(self, {__index = ClassStandardCrashbreaker})
    return self
end

------------------------------------------------------------------------------------------------------------------
-- Interface
----------------------------------------------------------------------------------------------------

return ClassStandardCrashbreaker