------------------------------------------------------------------------------------------------------------------
-- Imports
----------------------------------------------------------------------------------------------------

local ClassPickup               = require("scenario/intersection/classes/pickups/classPickup")
local classHelper               = require("scenario/intersection/classHelper")
local ClassStandardCrashbreaker = require("scenario/intersection/classes/crashbreaker/classStandard")

----------------------------------------------------------------------------------------------------

local ClassInstantCrashbreaker = classHelper.inheritsFrom(ClassPickup)

----------------------------------------------------------------------------------------------------
-- Static Attributes
----------------------------------------------------------------------------------------------------

ClassInstantCrashbreaker.className = "ClassInstantCrashbreaker"

----------------------------------------------------------------------------------------------------
-- Overridden Methods
-- See superclass for method descriptions
----------------------------------------------------------------------------------------------------

function ClassInstantCrashbreaker:collect(triggerVehicle)
    --[[
    See ClassPickup:collect()
    Creates and uses a crashbreaker explosion
    Parameters:
        triggerVehicle - the vehicle triggering the crashbreaker
    ]]--
    self.collected = true

    self.crashbreaker = ClassStandardCrashbreaker:new(self.vehicles, 0, false)
    self.crashbreaker:use(triggerVehicle)
end

function ClassInstantCrashbreaker:update(dt)
    ClassPickup.update(self, dt)

    if self.crashbreaker and self.crashbreaker:beingUsed() then
        self.crashbreaker:update(dt)
    end
end

function ClassInstantCrashbreaker:reset()
    ClassPickup.reset(self)

    if self.crashbreaker then
        self.crashbreaker:reset()
    end
end

----------------------------------------------------------------------------------------------------
-- Constructor
----------------------------------------------------------------------------------------------------

function ClassInstantCrashbreaker:new(position, name, vehicles)
    --[[
    Parameters:
        position - the position of the pickup
        name     - the name of this object in-world
        vehicles - all vehicles that can be affected by the crashbreaker
    ]]--
    local self = ClassPickup:new(position, "instant_crashbreaker", name)
    self.value = "crashbreaker"

    -- Lazy instantiation - may not always be needed
    self.crashbreaker = nil
    self.vehicles = vehicles

    setmetatable(self, {__index = ClassInstantCrashbreaker})
    return self
end

------------------------------------------------------------------------------------------------------------------
-- Interface
----------------------------------------------------------------------------------------------------

return ClassInstantCrashbreaker