local M = {}

local ClassVector = require("scenario/intersection/classes/classVector")
local ID = 0

local function generateTree(position, scale, model)
    --[[
    Generates a tree at a given position and scaling with random rotation
    Parameters:
        position - the position to create the tree at
        scale    - the xyz scale of the tree
        model    - the tree model to use
    ]]--
    TorqueScript.eval([[
      new TSStatic(tree_]]..ID..[[) {
        shapeName = "levels/smallgrid/art/intersection/trees/]]..model..[[.dae";
        scale = "]]..scale:getX()..[[ ]]..scale:getY()..[[ ]]..scale:getZ()..[[";
        position = "]]..position:getX()..[[ ]]..position:getY()..[[ ]]..position:getZ()..[[";
        rotation = "0 0 -1 ]]..(math.random()*360)..[[";
        collisionType = "Visible Mesh Final";
        decalType = "Visible Mesh Final";
      };

      ScenarioObjectsGroup.add(tree_]]..ID..[[);
    ]])

    ID = ID + 1
end

local function shuffleTrees(blacklist)
    --[[
    Randomly rotates all trees in the world
    Parameters:
        blacklist - a tree to blacklist
    ]]--
    local allObjs = scenetree.findClassObjects("TSStatic")

    for _, objName in ipairs(allObjs) do
        local obj = scenetree.findObject(objName)

        if obj.shapeName:find("tree") and not obj.shapeName:find(blacklist) then
            TorqueScript.eval(objName..'.rotation = "0 0 -1 '..(math.random()*360)..'";')
        end
    end
end

local function generateTrees(zCoord, startPos, endPos, prob, treeSpacing, treeScale, deviation, treeModel)
    --[[
    Generates trees in a rectangular area
    Parameters:
        zCoord       - the z plane coordinate for every tree
        startPos     - the starting position
        endPos       - the ending position (should be opposite corner of startPos)
                       This should have x and y coordinates greater than those of startPos
        prob         - the probability that a tree will be placed every step
        treeSpacing  - the minimum distance between trees
        treeScale    - the scale of each tree
        deviation    - the maximum deviation from the actual spot the tree will be placed at
        treeModel    - the tree model for the tree
    ]]--
    for x = startPos:getX(), endPos:getX(), treeSpacing do
        for y = startPos:getY(), endPos:getY(), treeSpacing do
            if math.random() <= prob then
                generateTree(ClassVector:new(x + math.random(-deviation, deviation), y + math.random(-deviation, deviation), zCoord), treeScale, treeModel)
            end
        end
    end

    print(ID)
end

M.generateTrees = generateTrees
M.shuffleTrees = shuffleTrees
return M