local M = {}

local function setupCloudCoverage(cloudState)
    --[[
    Setups the cloud layer for the scenario
    Parameters:
        cloudState - the cloud state (clear, cloudy, dense)
                     can be nil which will not create any cloud layer
    ]]--
    if not cloudState or cloudState == "clear" then
        -- Default to no clouds
        return
    end

    local cloudColour = ""
    local cloudCoverage = 0

    if cloudState == "cloudy" then
        cloudColour = "0.9 0.9 0.9 1"
        cloudCoverage = 1
    elseif cloudState == "dense" then
        cloudColour = "0.26 0.26 0.26 1"
        cloudCoverage = 3
    end

    -- Create the cloud layer
    TorqueScript.eval([[
        new CloudLayer(clouds) {
            baseColor = "]]..cloudColour  ..[[";
            coverage  = "]]..cloudCoverage..[[";
            texture = "art/skies/clouds/clouds_normal_displacement";
        };
    ]])
end

local function setupWeather(weatherState)
    --[[
    Setups the weather in the scenario
    Parameters:
        weatherState - the weather state (sunny, light-rain, heavy-rain, lightning)
                       can be nil which will set the weather to the default
    ]]--
    if not weatherState or weatherState == "sunny" then
        -- Default to standard scenario weather
        return
    end

    local numDrops = 0
    local dropSize = 0
    local datablock = ""

    if weatherState:find("rain") then
        if weatherState == "light-rain" then
            numDrops = 1024
            dropSize = 0.5
            setupCloudCoverage("cloudy")
        elseif weatherState == "heavy-rain" then
            numDrops = 4096
            dropSize = 1
            setupCloudCoverage("dense")
        end

        datablock = "rain_drop"
    elseif weatherState:find("snow") then
        if weatherState == "light-snow" then
            numDrops = 1024
            dropSize = 1
        elseif weatherState == "heavy-snow" then
            numDrops = 4096
            dropSize = 2
        end

        setupCloudCoverage("cloudy")
        datablock = "Snow_menu"
    end

    -- Create the precipitation object
    TorqueScript.eval([[
        new Precipitation(weather) {
            numDrops  = "]]..numDrops ..[[";
            dropSize  = "]]..dropSize ..[[";
            datablock = "]]..datablock..[[";
        };
    ]])
end

local function setupTime(tod)
    --[[
    Set the time in the scenario
    Parameters:
        tod - the time of day to set (morning, noon, midnight)
              can be nil to go to default scenario time
    ]]--
    if not tod or tod == "noon" then
        return
    end

    local time = 0

    if tod == "morning" then
        time = 0.19
    elseif tod == "midnight" then
        time = 0.35
    end

    scenetree.findObject("tod").time = time
end

M.setupWeather = setupWeather
M.setupTime = setupTime
return M